<?php

/**
 * UṔgates API client
 * @copyright  Copyright (c) 2020 EVici webdesign s.r.o.
 * @link       https://www.upgates.cz/api-v2
 */

namespace UPgates\Api;

class Client
{

	/**
	 * Request HTTP method
	 *
	 * @var string
	 */
	private $method;

	/**
	 * Request URL
	 *
	 * @var string
	 */
	private $url;

	/**
	 * URL parameters
	 *
	 * @var array
	 */
	private $params = [];

	/**
	 * Set URL params
	 *
	 * @param array $params
	 */
	public function setParams(array $params)
	{
		$this->params = $params;
	}

	/**
	 * Request HTTP headers
	 *
	 * @var array
	 */
	private $headers = [];

	/**
	 * Request body
	 *
	 * @var string
	 */
	private $body;

	/**
	 * Constructor
	 *
	 * @param string $method	HTTP mathod
	 * @param string $url		URL
	 * @param array $params		URL query params
	 * @param array $headers	HTTP headers
	 */
	public function __construct(string $method, string $url, array $params = [], array $headers = [])
	{
		if (!in_array(strtoupper($method), ['DELETE', 'GET', 'PUT', 'POST'])) {
			throw new \InvalidArgumentException("Unsupported HTTP method $method !");
		}

		$this->method = $method;
		$this->url = $url;
		$this->params = $params;
		$this->headers = $headers;
	}

	/**
	 * Set authentication
	 *
	 * @param string $login
	 * @param string $password
	 */
	public function setAuth(string $login, string $password)
	{
		$this->headers['Authorization'] = 'Basic ' . base64_encode("$login:$password");
	}

	/**
	 * Set body
	 *
	 * @param string $body
	 * @param string $contentType
	 */
	public function setBody(string $body, string $contentType = 'application/json')
	{
		$this->headers['Content-Type'] = $contentType;
		$this->body = $body;
	}

	/**
	 * Get URI
	 *
	 * @return string
	 */
	public function getUri(): string
	{
		return $this->url . ($this->params ? '?' . http_build_query($this->params) : '');
	}

	/**
	 * Call API
	 *
	 * @return \stdClass
	 */
	public function call(): \stdClass
	{
		/* CURL init */
		$curl = curl_init($this->getUri());

		/* set options */
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curl, CURLOPT_FOLLOWLOCATION, 1);
		curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $this->method);
		curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 60);
		curl_setopt($curl, CURLOPT_TIMEOUT, 300);
		curl_setopt($curl, CURLOPT_HEADER, 1);

		/* set headers */
		if ($this->headers) {
			$headers = [];
			array_walk($this->headers, function ($value, $key) use (&$headers) {
				$headers[] = "$key: $value";
			});
			curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
		}

		/* set body */
		if ($this->body) {
			curl_setopt($curl, CURLOPT_POSTFIELDS, $this->body);
		}

		/* get response */
		$body = curl_exec($curl);

		/* HTTP status code */
		$status = curl_getinfo($curl, CURLINFO_HTTP_CODE);

		/* response headers */
		$headers = [];

		/* body is ok */
		if ($body !== FALSE) {

			/* get headers from body */
			foreach (explode("\n", trim(substr($body, 0, curl_getinfo($curl, CURLINFO_HEADER_SIZE)))) as $header) {
				$pos = strpos($header, ':');
				if ($pos) {
					$name = substr($header, 0, $pos);
					$value = trim(substr($header, $pos + 1));
					$headers[$name] = $value;
				}
			}

			/* remove headers from body */
			$body = substr($body, curl_getinfo($curl, CURLINFO_HEADER_SIZE));

			/* get Content-Type header */
			$contentType = isset($headers['Content-Type']) ? isset($headers['Content-Type']) : null;

			/* if content is JSON, decode it */
			if ($contentType == 'application/json') {
				$data = @json_decode($body);
				if (json_last_error() == 0) {
					$body = $data;
				}
			}
		}
		/* error */
		else {
			$contentType = null;
			$body = curl_error($curl) . '(#' . curl_errno($curl) . ')';
			$status = null;
		}

		/* close CURL */
		curl_close($curl);

		/* result */
		return (object) [
					'contentType' => $contentType,
					'body' => $body,
					'status' => $status,
					'headers' => $headers
		];
	}

}
